/*
 * Copyright (c) 2013 J. Lewis Muir <jlmuir@imca-cat.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package imcacat.procrun;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.util.Arrays;

import org.junit.Test;

/**
 * I test {@link GrowingBufferInputStreamConsumer}.
 */
public class GrowingBufferInputStreamConsumerTest extends ProcrunTest {
  @Test
  public void testNoInput() throws InterruptedException {
    ByteArrayInputStream stream = new ByteArrayInputStream(new byte[0]);
    GrowingBufferInputStreamConsumer consumer = new GrowingBufferInputStreamConsumer();
    consumer.stream(stream);
    consumer.start();
    consumer.join();
    assertFalse(consumer.hasException());
    byte[] expected = new byte[0];
    byte[] actual = byteBufferToArray(consumer.data());
    assertTrue(Arrays.equals(expected, actual));
  }

  @Test
  public void testWithInput() throws InterruptedException {
    byte[] expected = "abc".getBytes();
    ByteArrayInputStream stream = new ByteArrayInputStream(expected);
    GrowingBufferInputStreamConsumer consumer = new GrowingBufferInputStreamConsumer(128);
    consumer.stream(stream);
    consumer.start();
    consumer.join();
    assertFalse(consumer.hasException());
    byte[] actual = byteBufferToArray(consumer.data());
    assertTrue(Arrays.equals(expected, actual));
  }

  @Test
  public void testInputBiggerThanInitialBuffer() throws InterruptedException {
    byte[] expected = "These aren't the droids you're looking for.".getBytes();
    ByteArrayInputStream stream = new ByteArrayInputStream(expected);
    GrowingBufferInputStreamConsumer consumer = new GrowingBufferInputStreamConsumer(8);
    consumer.stream(stream);
    consumer.start();
    consumer.join();
    assertFalse(consumer.hasException());
    byte[] actual = byteBufferToArray(consumer.data());
    assertTrue(Arrays.equals(expected, actual));
  }

  @Test
  public void testInputSizeOneByteLessThanInitialBuffer() throws InterruptedException {
    byte[] expected = "abc".getBytes();
    ByteArrayInputStream stream = new ByteArrayInputStream(expected);
    GrowingBufferInputStreamConsumer consumer =
        new GrowingBufferInputStreamConsumer(expected.length + 1);
    consumer.stream(stream);
    consumer.start();
    consumer.join();
    assertFalse(consumer.hasException());
    assertEquals(expected.length + 1, consumer.data().capacity());
  }

  @Test
  public void testInputSizeEqualToInitialBuffer() throws InterruptedException {
    byte[] expected = "abc".getBytes();
    ByteArrayInputStream stream = new ByteArrayInputStream(expected);
    GrowingBufferInputStreamConsumer consumer =
        new GrowingBufferInputStreamConsumer(expected.length);
    consumer.stream(stream);
    consumer.start();
    consumer.join();
    assertFalse(consumer.hasException());
    assertEquals(expected.length * 2, consumer.data().capacity());
  }
}
