/*
 * Copyright (c) 2013 J. Lewis Muir <jlmuir@imca-cat.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package imcacat.procrun;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.nio.ByteBuffer;

import org.junit.Test;

/**
 * I test {@link ProcessRunnerResult}.
 */
public class ProcessRunnerResultTest extends ProcrunTest {
  @Test
  public void testHasExitStatus() {
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, null);
    assertFalse(result.hasExitStatus());
    result = new ProcessRunnerResult(1, null, null, null);
    assertTrue(result.hasExitStatus());
  }

  @Test
  public void testExitStatus() {
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, null);
    assertNull(result.exitStatus());
    result = new ProcessRunnerResult(1, null, null, null);
    assertEquals(Integer.valueOf(1), result.exitStatus());
    result = new ProcessRunnerResult(2, null, null, null);
    assertEquals(Integer.valueOf(2), result.exitStatus());
  }

  @Test
  public void testHasException() {
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, null);
    assertFalse(result.hasException());
    result = new ProcessRunnerResult(null, new RuntimeException(), null, null);
    assertTrue(result.hasException());
  }

  @Test
  public void testException() {
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, null);
    assertNull(result.exception());
    Exception expected = new RuntimeException("test");
    result = new ProcessRunnerResult(null, expected, null, null);
    assertEquals(expected, result.exception());
  }

  @Test
  public void testHasStdout() {
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, null);
    assertFalse(result.hasStdout());
    result = new ProcessRunnerResult(null, new RuntimeException(), ByteBuffer.allocate(0), null);
    assertTrue(result.hasStdout());
  }

  @Test
  public void testStdout() {
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, null);
    assertNull(result.stdout());
    ByteBuffer expected = ByteBuffer.allocate(0);
    result = new ProcessRunnerResult(null, null, expected, null);
    assertEquals(expected, result.stdout());
  }

  @Test
  public void testStdoutString() {
    String expected = "abc";
    ByteBuffer buffer = ByteBuffer.wrap(expected.getBytes(US_ASCII));
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, buffer, null);
    assertEquals(expected, result.stdoutString(US_ASCII));
  }

  @Test
  public void testHasStderr() {
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, null);
    assertFalse(result.hasStderr());
    result = new ProcessRunnerResult(null, new RuntimeException(), null, ByteBuffer.allocate(0));
    assertTrue(result.hasStderr());
  }

  @Test
  public void testStderr() {
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, null);
    assertNull(result.stderr());
    ByteBuffer expected = ByteBuffer.allocate(0);
    result = new ProcessRunnerResult(null, null, null, expected);
    assertEquals(expected, result.stderr());
  }

  @Test
  public void testStderrString() {
    String expected = "abc";
    ByteBuffer buffer = ByteBuffer.wrap(expected.getBytes(US_ASCII));
    ProcessRunnerResult result = new ProcessRunnerResult(null, null, null, buffer);
    assertEquals(expected, result.stderrString(US_ASCII));
  }
}
